/*
 *  GUI/Sidebar/Sidebar.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using System.Collections;
using GLib;
using Gtk;
using Mono.Unix;


namespace Incollector.GUI {
	
	public class Sidebar : Gtk.VBox {
		
		protected Gtk.Label        Title;
		protected Gtk.Menu         SidebarMenu;
		protected Gtk.Notebook     SidebarNotebook;
		protected Gtk.ToggleButton MenuButton;
		protected ArrayList        MenuItems = new ArrayList ();
		
		public delegate void CloseBtnClickedHandler ();
		protected CloseBtnClickedHandler CloseBtnClicked;
		
		
		public Sidebar (CloseBtnClickedHandler OnCloseBtnClicked)
		{
			this.CloseBtnClicked = OnCloseBtnClicked;
			this.Spacing = 4;
			
			
			Gtk.HBox TitleBox = new Gtk.HBox ();
			TitleBox.Spacing = 3;
			this.PackStart (TitleBox, false, false, 0);
			
			MenuButton = new Gtk.ToggleButton ();
			MenuButton.Toggled += new EventHandler (on_MenuButton_toggled);
			MenuButton.Relief = ReliefStyle.None;
			TitleBox.PackStart (MenuButton, true, true, 0);
			
			Gtk.HBox MenuButtonBox = new Gtk.HBox ();
			MenuButtonBox.Spacing = 3;
			MenuButton.Add (MenuButtonBox);
			
			Title = new Gtk.Label ();
			Title.Xalign = (float) 0.00;
			MenuButtonBox.PackStart (Title, true, true, 0);
			
			Gtk.Arrow arrow = new Gtk.Arrow (ArrowType.Down, ShadowType.Out);
			MenuButtonBox.PackEnd (arrow, false, false, 0);
			
			Gtk.Button CloseBtn = new Gtk.Button ();
			CloseBtn.Clicked += new EventHandler (on_CloseBtn_clicked);
			CloseBtn.Relief = Gtk.ReliefStyle.None;
			CloseBtn.Image = new Gtk.Image (Gtk.Stock.Close, Gtk.IconSize.Menu);
			TitleBox.PackEnd (CloseBtn, false, false, 0);
			
			
			SidebarNotebook = new Gtk.Notebook ();
			SidebarNotebook.ShowTabs = false;
			SidebarNotebook.ShowBorder = false;
			this.PackEnd (SidebarNotebook, true, true, 0);
			
			
			SidebarMenu = new Menu ();
			SidebarMenu.Deactivated += new EventHandler (on_SidebarMenu_deactivated);
			SidebarMenu.Show ();
			
			
			this.ShowAll ();
		}
		
		
		public void AddWidget (Gtk.Widget widget, string title)
		{
			Gtk.MenuItem item = new Gtk.MenuItem (title);
			item.Activated += new EventHandler (on_SidebarMenuItem_activated);
			item.Name = SidebarNotebook.NPages.ToString () + ";" + title;
			SidebarMenu.Append (item);
			item.Show ();
			
			MenuItems.Add (item);
			SidebarNotebook.Add (widget);
			
			if (SidebarNotebook.NPages == 1)
				item.Activate ();
		}
		
		
		public void ActivateWidget (Gtk.Widget widget)
		{
			int number = -1;
			
			for (int i = 0; i < this.SidebarNotebook.NPages; i++)
				if (this.SidebarNotebook.GetNthPage (i) == widget) {
					number = i;
					break;
				}
			
			if (number != -1)
				(this.MenuItems [number] as Gtk.MenuItem).Activate ();
		}
		
		
		protected void on_SidebarMenuItem_activated (object o, EventArgs args)
		{
			string tmp = (o as MenuItem).Name;
			
			try {
				Title.Text = tmp.Substring (tmp.IndexOf (";") + 1);
				SidebarNotebook.Page = Convert.ToInt32 (tmp.Substring (0, tmp.IndexOf (";")));
			} catch {}
		}
		
		
		protected void on_SidebarMenu_deactivated (object o, EventArgs args)
		{
			MenuButton.Active = false;
		}
		
		
		protected void SidebarMenuPositionFunc (Menu menu, out int x, out int y, out bool push_in)
		{
			SidebarMenu.WidthRequest = MenuButton.Allocation.Width;
			
			MenuButton.GdkWindow.GetOrigin (out x, out y);
			x += MenuButton.Allocation.X;
			y += MenuButton.Allocation.Y + MenuButton.Allocation.Height;
			push_in = true;
		}
		
		
		protected void on_MenuButton_toggled (object o, EventArgs args)
		{
			if ((o as Gtk.ToggleButton).Active)
				SidebarMenu.Popup (null, null, SidebarMenuPositionFunc, 0, 0);
		}
		
		
		protected void on_CloseBtn_clicked (object o, EventArgs args)
		{
			if (CloseBtnClicked != null)
				CloseBtnClicked ();
		}
	}
}
