/*
 *  GUI/EntryPreview/EntryPreview.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Reflection;
using System.Collections;
using Mono.Unix;
using Gtk;
using Gdk;
using Incollector.GUI;
using Incollector.GUI.Widgets;
using Incollector.Engine;


namespace Incollector.GUI.Widgets {

	public class EntryPreview : Gtk.ScrolledWindow {
		
		protected Gtk.Label      MessageLabel, DateLabel, CategoryLabel, TagsLabel, TitleLabel;
		protected PreviewLabel   Comment;
		protected Gtk.Image      RatingImage, CategoryIcon;
		protected Gtk.Notebook   Pages;
		protected Gtk.VBox       PreviewBox;
		protected Gtk.HSeparator CommentSeparator;
		protected Gtk.Viewport   CommentViewport;
		
		protected int _EntryID;
		
		protected string MessageUnavailable    = Catalog.GetString ("No entry to preview, select an entry from the entries list to preview it.");
		protected string MessageLoadingPreview = Catalog.GetString ("Loading preview...");
		
		
		protected enum PagesEnum {
			Message = 0,
			Preview = 1
		}
		
		
		public int EntryID {
			get { return _EntryID; }
		}
		
		protected int Rating {
			set {
				if ((value >= 0) && (value <= 3))
					Icons.AssignImage ("stars-" + value + ".png", RatingImage);
				else
					Icons.AssignImage ("stars-0.png", RatingImage);
			}
		}
		
		
		
		public EntryPreview ()
		{
			_EntryID = -1;
			
			// style hack
			Gtk.Window wdw = new Gtk.Window ("");
			wdw.EnsureStyle ();
			
			this.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic);
			this.ShadowType = Gtk.ShadowType.In;
			
			Gtk.Viewport MainBox = new Gtk.Viewport ();
			Colors.Modify (wdw.Style.BaseColors[0], MainBox);
			MainBox.ShadowType = Gtk.ShadowType.None;
			this.Add (MainBox);
			
			Pages = new Gtk.Notebook ();
			Pages.ShowBorder = false;
			Pages.ShowTabs   = false;
			MainBox.Add (Pages);
			
			MessageLabel = new Gtk.Label ();
			MessageLabel.Text = MessageUnavailable;
			Pages.Add (MessageLabel);
			
			Gtk.VBox vbox1 = new Gtk.VBox ();
			Pages.Add (vbox1);
			
			
			PreviewBox = new Gtk.VBox ();
			PreviewBox.BorderWidth = 8;
			vbox1.PackStart (PreviewBox, true, true, 0);
			
			
			CommentSeparator = new Gtk.HSeparator ();
			vbox1.PackStart (CommentSeparator, false, false, 0);
			
			
			CommentViewport = new Gtk.Viewport ();
			CommentViewport.ShadowType = Gtk.ShadowType.None;
			Colors.Modify (wdw.Style.Backgrounds[2], CommentViewport);
			vbox1.PackStart (CommentViewport, false, false, 0);
			
			Gtk.VBox CommentBox = new Gtk.VBox ();
			CommentBox.BorderWidth = 8;
			CommentViewport.Add (CommentBox);
			
			Gtk.Label CommentCaption = new Gtk.Label ();
			CommentCaption.Xalign = (float) 0.00;
			CommentCaption.UseMarkup = true;
			CommentCaption.Markup = "<b>" + Catalog.GetString ("Comment") + "</b>";
			CommentBox.PackStart (CommentCaption, false, false, 0);
			
			Gtk.Alignment calign = new Gtk.Alignment ((float) 0.00, (float) 0.00, 1, 1);
			calign.LeftPadding = 12;
			CommentBox.PackEnd (calign, false, false, 3);
			
			Comment = new PreviewLabel ("");
			Comment.Wrap = true;
			Comment.Width = 500;
			Comment.Xalign = (float) 0.00;
			Comment.Text = "";
			calign.Add (Comment);
			
			
			Gtk.HSeparator separator2 = new Gtk.HSeparator ();
			vbox1.PackStart (separator2, false, false, 0);
			
			
			Gtk.Viewport InfoViewport = new Gtk.Viewport ();
			InfoViewport.ShadowType = Gtk.ShadowType.None;
			Colors.Modify (wdw.Style.Backgrounds[2], InfoViewport);
			vbox1.PackEnd (InfoViewport, false, false, 0);
			
			Gtk.VBox InfoBox = new Gtk.VBox ();
			InfoBox.Spacing = 8;
			InfoBox.BorderWidth = 8;
			InfoViewport.Add (InfoBox);
			
			
			Gtk.HBox EntryInfoBox = new Gtk.HBox ();
			InfoBox.PackEnd (EntryInfoBox, false, false, 0);
			EntryInfoBox.Spacing = 6;
			
			Gtk.Image DateIcon = new Gtk.Image ();
			Icons.AssignImage ("date-16.png", DateIcon);
			CategoryIcon = new Gtk.Image ();
			Gtk.Image TagsIcon = new Gtk.Image ();
			Icons.AssignImage ("tags-16.png", TagsIcon);
			RatingImage = new Gtk.Image ();
			
			DateLabel     = new Gtk.Label ();
			CategoryLabel = new Gtk.Label ();
			TagsLabel     = new Gtk.Label ();
			TagsLabel.Xalign = (float) 0.00;
			
			EntryInfoBox.PackStart (DateIcon, false, false, 0);
			EntryInfoBox.PackStart (DateLabel, false, false, 2);
			EntryInfoBox.PackStart (new Gtk.Fixed (), false, false, 2);
			EntryInfoBox.PackStart (CategoryIcon, false, false, 2);
			EntryInfoBox.PackStart (CategoryLabel, false, false, 0);
			EntryInfoBox.PackStart (new Gtk.Fixed (), false, false, 2);
			EntryInfoBox.PackStart (TagsIcon, false, false, 2);
			EntryInfoBox.PackStart (TagsLabel, true, true, 0);
			EntryInfoBox.PackStart (RatingImage, false, false, 0);
			
			
			this.ShowAll ();
		}
		
		
		public void LoadPreview (Engine.Entry entry)
		{
			_EntryID = entry.ID;
			
			if (entry.ID == -1) { // if entry doesn't exist
				Pages.Page = (int) PagesEnum.Message;
				MessageLabel.Text = MessageUnavailable;
				
				this.Comment.Text = "";
				
				if (PreviewBox.Children.Length > 0)
					PreviewBox.Remove (PreviewBox.Children [0]);
			} else {
				Pages.Page = (int) PagesEnum.Message;
				MessageLabel.Text = MessageLoadingPreview;
				Tools.Repaint ();
				
				if (PreviewBox.Children.Length > 0)
					PreviewBox.Remove (PreviewBox.Children [0]);
				
				PreviewBox.PackStart (entry.PreviewWidget);
				Comment.Text = entry.Comment;
				
				
				this.CategoryIcon.Pixbuf = entry.Data.Icon16;
				this.Rating = entry.Rating;
				this.DateLabel.Text = entry.GetDateString (Engine.Settings.GetBool ("DateFirstYear"));
				this.CategoryLabel.Text = entry.Category;
				this.TagsLabel.Text = entry.GetTagsString ();
				
				CommentViewport.Visible = CommentSeparator.Visible = Comment.Text != "";
				
				Pages.Page = (int) PagesEnum.Preview;
			}
		}
	}
}
