/*
 *  Searching/Queries/Comment.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Collections;
using Mono.Unix;
using System.Xml;
using Incollector.Engine;
using Incollector.GUI.Widgets;


namespace Incollector.Searching.Queries {
	
	public class Comment : Incollector.Searching.Query {
		
		protected Gtk.Entry    comment;
		protected Gtk.Widget   widget;
		protected Gtk.ComboBox conditionals;
		
		public enum ConditionalEnum {
			Contain    = 0,
			NotContain = 1
		}
		
		protected ConditionalEnum Conditional {
			get { return (ConditionalEnum) conditionals.Active; }
			set { conditionals.Active = (int) value; }
		}
		
		protected string CommentValue {
			get { return this.comment.Text;  }
			set { this.comment.Text = value; }
		}
		
		public override Gtk.Widget QueryWidget {
			get { return widget; }
		}
		
		public override string ID {
			get { return "comment"; }
		}
		
		public override string Name {
			get { return Catalog.GetString ("Comment"); }
		}
		
		
		public Comment ()
		{
			widget = new Gtk.HBox ();
			(widget as Gtk.HBox).Spacing = 6;
			
			conditionals = new Gtk.ComboBox (new string [] {
								Catalog.GetString ("contains"),
								Catalog.GetString ("doesn't contain")
							});
			conditionals.Active = 0;
			(widget as Gtk.HBox).PackStart (conditionals, true, true, 0);
			
			comment = new Gtk.Entry ();
			(widget as Gtk.HBox).PackEnd (comment, true, true, 0);
			
			widget.ShowAll ();
		}
		
		
		public override Query New ()
		{
			return (Query) new Comment ();
		}
		
		
		public override Query Clone ()
		{
			Comment query = new Comment ();
			query.Conditional  = this.Conditional;
			query.CommentValue = this.CommentValue.Clone () as string;
			
			return query;
		}
		
		
		public override bool CheckQuery (Engine.Entry entry)
		{	
			if (this.Conditional == ConditionalEnum.Contain)
				return Text.Search (this.CommentValue, entry.Comment);
			else
				return !Text.Search (this.CommentValue, entry.Comment);
		}
		
		
		public override Query ReadXml (XmlTextReader reader)
		{
			Comment query = new Comment ();
			
			try {
				query.Conditional = (ConditionalEnum) System.Enum.Parse (typeof (ConditionalEnum), reader.GetAttribute ("conditional"));
			} catch {}
			
			query.CommentValue = Text.PrepareText2 (reader.GetAttribute ("comment"));
			
			return query;
		}
		
		
		public override void WriteXml (XmlTextWriter writer)
		{
			writer.WriteStartElement (this.ID);
				writer.WriteAttributeString ("conditional", Conditional.ToString ());
				writer.WriteAttributeString ("comment", Text.PrepareText (this.CommentValue));
			writer.WriteEndElement ();
		}
	}
}
