/*
 *  Searching/Queries/Tags.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Collections;
using Mono.Unix;
using System.Xml;
using Incollector.Engine;
using Incollector.GUI.Widgets;


namespace Incollector.Searching.Queries {
	
	public class Tags : Incollector.Searching.Query {
		
		protected TagsEntry    tag;
		protected Gtk.Widget   widget;
		protected Gtk.ComboBox conditionals;
		
		public enum ConditionalEnum {
			Contain    = 0,
			NotContain = 1
		}
		
		protected ConditionalEnum Conditional {
			get { return (ConditionalEnum) conditionals.Active; }
			set { conditionals.Active = (int) value; }
		}
		
		protected ArrayList TagsList {
			get { return tag.Tags;  }
			set { tag.Tags = value; }
		}
		
		public override Gtk.Widget QueryWidget {
			get { return widget; }
		}
		
		public override string ID {
			get { return "tags"; }
		}
		
		public override string Name {
			get { return Catalog.GetString ("Tags"); }
		}
		
		
		public Tags ()
		{
			widget = new Gtk.HBox ();
			(widget as Gtk.HBox).Spacing = 6;
			
			conditionals = new Gtk.ComboBox (new string [] {
								Catalog.GetString ("contain"),
								Catalog.GetString ("don't contain")
							});
			conditionals.Active = 0;
			(widget as Gtk.HBox).PackStart (conditionals, true, true, 0);
			
			if (Globals.Inited)
				tag = new TagsEntry (Globals.Tags.Tags);
			else
				tag = new TagsEntry (new string [] {""});
			
			(widget as Gtk.HBox).PackEnd (tag, true, true, 0);
			
			widget.ShowAll ();
		}
		
		
		public override Query New ()
		{
			return (Query) new Tags ();
		}
		
		
		public override Query Clone ()
		{
			Tags query = new Tags ();
			query.Conditional = this.Conditional;
			query.TagsList = this.TagsList.Clone () as ArrayList;
			
			return query;
		}
		
		
		public override bool CheckQuery (Engine.Entry entry)
		{
			foreach (string tag in TagsList)
				if (this.Conditional == ConditionalEnum.Contain) {
					if (entry.TagExists (tag) == false)
						return false;
				} else {
					if (entry.TagExists (tag) == true)
						return false;
				}
			
			return true;
		}
		
		
		public override Query ReadXml (XmlTextReader reader)
		{
			Tags query = new Tags ();
			
			try {
				query.Conditional = (ConditionalEnum) System.Enum.Parse (typeof (ConditionalEnum), reader.GetAttribute ("conditional"));
			} catch {}
			
			string tags = reader.GetAttribute ("tags");
			
			try {
				ArrayList list = new ArrayList ();
				foreach (string tag in tags.Split (','))
					if (tag != String.Empty)
						list.Add (Text.PrepareText2 (tag));
				query.TagsList = list;
			} catch {}
			
			return query;
		}
		
		
		public override void WriteXml (XmlTextWriter writer)
		{
			writer.WriteStartElement (this.ID);
				writer.WriteAttributeString ("conditional", Conditional.ToString ());
				
				string tags = "";
				foreach (string tag in this.TagsList)
					tags += Text.PrepareText (tag) + ",";
				
				writer.WriteAttributeString ("tags", tags);
				
			writer.WriteEndElement ();
		}
	}
}
